@@ -27,6 +27,7 @@
  */
 // Sub modules
 mod bookmarks;
+mod components;
 mod misc;
 mod update;
 mod view;
@@ -36,34 +37,131 @@ use super::{Activity, Context, ExitReason};
 use crate::config::themes::Theme;
 use crate::filetransfer::{FileTransferParams, FileTransferProtocol};
 use crate::system::bookmarks_client::BookmarksClient;
-use crate::utils::git;
+use crate::system::config_client::ConfigClient;
 
 // Includes
-use crossterm::event::Event;
-use crossterm::terminal::{disable_raw_mode, enable_raw_mode};
-use tuirealm::{Update, View};
+use std::time::Duration;
+use tuirealm::listener::EventListenerCfg;
+use tuirealm::{application::PollStrategy, Application, NoUserEvent, Update};
 
 // -- components
-const COMPONENT_TEXT_H1: &str = "TEXT_H1";
-const COMPONENT_TEXT_H2: &str = "TEXT_H2";
-const COMPONENT_TEXT_NEW_VERSION: &str = "TEXT_NEW_VERSION";
-const COMPONENT_TEXT_NEW_VERSION_NOTES: &str = "TEXTAREA_NEW_VERSION";
-const COMPONENT_TEXT_FOOTER: &str = "TEXT_FOOTER";
-const COMPONENT_TEXT_HELP: &str = "TEXT_HELP";
-const COMPONENT_TEXT_ERROR: &str = "TEXT_ERROR";
-const COMPONENT_TEXT_SIZE_ERR: &str = "TEXT_SIZE_ERR";
-const COMPONENT_INPUT_ADDR: &str = "INPUT_ADDRESS";
-const COMPONENT_INPUT_PORT: &str = "INPUT_PORT";
-const COMPONENT_INPUT_USERNAME: &str = "INPUT_USERNAME";
-const COMPONENT_INPUT_PASSWORD: &str = "INPUT_PASSWORD";
-const COMPONENT_INPUT_BOOKMARK_NAME: &str = "INPUT_BOOKMARK_NAME";
-const COMPONENT_RADIO_PROTOCOL: &str = "RADIO_PROTOCOL";
-const COMPONENT_RADIO_QUIT: &str = "RADIO_QUIT";
-const COMPONENT_RADIO_BOOKMARK_DEL_BOOKMARK: &str = "RADIO_DELETE_BOOKMARK";
-const COMPONENT_RADIO_BOOKMARK_DEL_RECENT: &str = "RADIO_DELETE_RECENT";
-const COMPONENT_RADIO_BOOKMARK_SAVE_PWD: &str = "RADIO_SAVE_PASSWORD";
-const COMPONENT_BOOKMARKS_LIST: &str = "BOOKMARKS_LIST";
-const COMPONENT_RECENTS_LIST: &str = "RECENTS_LIST";
+#[derive(Debug, Eq, PartialEq, Clone, Hash)]
+pub enum Id {
+    Address,
+    BookmarkName,
+    BookmarkSavePassword,
+    BookmarksList,
+    DeleteBookmarkPopup,
+    DeleteRecentPopup,
+    ErrorPopup,
+    GlobalListener,
+    HelpFooter,
+    InfoPopup,
+    InstallUpdatePopup,
+    Keybindings,
+    NewVersionChangelog,
+    NewVersionDisclaimer,
+    Password,
+    Port,
+    Protocol,
+    QuitPopup,
+    RecentsList,
+    S3AccessKey,
+    S3Bucket,
+    S3Endpoint,
+    S3NewPathStyle,
+    S3Profile,
+    S3Region,
+    S3SecretAccessKey,
+    S3SecurityToken,
+    S3SessionToken,
+    Subtitle,
+    Title,
+    Username,
+    WaitPopup,
+    WindowSizeError,
+}
+
+#[derive(Debug, PartialEq)]
+pub enum Msg {
+    Form(FormMsg),
+    Ui(UiMsg),
+    None,
+}
+
+#[derive(Debug, PartialEq)]
+pub enum FormMsg {
+    Connect,
+    DeleteBookmark,
+    DeleteRecent,
+    EnterSetup,
+    InstallUpdate,
+    LoadBookmark(usize),
+    LoadRecent(usize),
+    ProtocolChanged(FileTransferProtocol),
+    Quit,
+    SaveBookmark,
+}
+
+#[derive(Debug, PartialEq)]
+pub enum UiMsg {
+    AddressBlurDown,
+    AddressBlurUp,
+    BookmarksListBlur,
+    BookmarksTabBlur,
+    CloseDeleteBookmark,
+    CloseDeleteRecent,
+    CloseErrorPopup,
+    CloseInfoPopup,
+    CloseInstallUpdatePopup,
+    CloseKeybindingsPopup,
+    CloseQuitPopup,
+    CloseSaveBookmark,
+    ParamsFormBlur,
+    PasswordBlurDown,
+    PasswordBlurUp,
+    PortBlurDown,
+    PortBlurUp,
+    ProtocolBlurDown,
+    ProtocolBlurUp,
+    RececentsListBlur,
+    S3AccessKeyBlurDown,
+    S3AccessKeyBlurUp,
+    S3BucketBlurDown,
+    S3BucketBlurUp,
+    S3EndpointBlurDown,
+    S3EndpointBlurUp,
+    S3NewPathStyleBlurDown,
+    S3NewPathStyleBlurUp,
+    S3ProfileBlurDown,
+    S3ProfileBlurUp,
+    S3RegionBlurDown,
+    S3RegionBlurUp,
+    S3SecretAccessKeyBlurDown,
+    S3SecretAccessKeyBlurUp,
+    S3SecurityTokenBlurDown,
+    S3SecurityTokenBlurUp,
+    S3SessionTokenBlurDown,
+    S3SessionTokenBlurUp,
+    BookmarkNameBlur,
+    SaveBookmarkPasswordBlur,
+    ShowDeleteBookmarkPopup,
+    ShowDeleteRecentPopup,
+    ShowKeybindingsPopup,
+    ShowQuitPopup,
+    ShowReleaseNotes,
+    ShowSaveBookmarkPopup,
+    UsernameBlurDown,
+    UsernameBlurUp,
+    WindowResized,
+}
+
+/// Auth form input mask
+#[derive(Eq, PartialEq)]
+enum InputMask {
+    Generic,
+    AwsS3,
+}
 
 // Store keys
 const STORE_KEY_LATEST_VERSION: &str = "AUTH_LATEST_VERSION";
@@ -73,101 +171,72 @@ const STORE_KEY_RELEASE_NOTES: &str = "AUTH_RELEASE_NOTES";
 ///
 /// AuthActivity is the data holder for the authentication activity
 pub struct AuthActivity {
-    exit_reason: Option<ExitReason>,
-    context: Option<Context>,
-    view: View,
+    app: Application<Id, Msg, NoUserEvent>,
     bookmarks_client: Option<BookmarksClient>,
-    redraw: bool,                // Should ui actually be redrawned?
-    bookmarks_list: Vec<String>, // List of bookmarks
-    recents_list: Vec<String>,   // list of recents
-}
-
-impl Default for AuthActivity {
-    fn default() -> Self {
-        Self::new()
-    }
+    /// List of bookmarks
+    bookmarks_list: Vec<String>,
+    /// List of recent hosts
+    recents_list: Vec<String>,
+    /// Exit reason
+    exit_reason: Option<ExitReason>,
+    /// Should redraw ui
+    redraw: bool,
+    /// Protocol
+    protocol: FileTransferProtocol,
+    context: Option<Context>,
 }
 
 impl AuthActivity {
-    /// ### new
-    ///
     /// Instantiates a new AuthActivity
-    pub fn new() -> AuthActivity {
+    pub fn new(ticks: Duration) -> AuthActivity {
         AuthActivity {
-            exit_reason: None,
+            app: Application::init(
+                EventListenerCfg::default()
+                    .default_input_listener(ticks)
+                    .poll_timeout(ticks),
+            ),
             context: None,
-            view: View::init(),
             bookmarks_client: None,
-            redraw: true, // True at startup
             bookmarks_list: Vec::new(),
+            exit_reason: None,
             recents_list: Vec::new(),
+            redraw: true,
+            protocol: FileTransferProtocol::Sftp,
         }
     }
 
-    /// ### on_create
-    ///
-    /// If enabled in configuration, check for updates from Github
-    fn check_for_updates(&mut self) {
-        debug!("Check for updates...");
-        // Check version only if unset in the store
-        let ctx: &mut Context = self.context_mut();
-        if !ctx.store().isset(STORE_KEY_LATEST_VERSION) {
-            debug!("Version is not set in storage");
-            if ctx.config().get_check_for_updates() {
-                debug!("Check for updates is enabled");
-                // Send request
-                match git::check_for_updates(env!("CARGO_PKG_VERSION")) {
-                    Ok(Some(git::GithubTag { tag_name, body })) => {
-                        // If some, store version and release notes
-                        info!("Latest version is: {}", tag_name);
-                        ctx.store_mut()
-                            .set_string(STORE_KEY_LATEST_VERSION, tag_name);
-                        ctx.store_mut().set_string(STORE_KEY_RELEASE_NOTES, body);
-                    }
-                    Ok(None) => {
-                        info!("Latest version is: {} (current)", env!("CARGO_PKG_VERSION"));
-                        // Just set flag as check
-                        ctx.store_mut().set(STORE_KEY_LATEST_VERSION);
-                    }
-                    Err(err) => {
-                        // Report error
-                        error!("Failed to get latest version: {}", err);
-                        self.mount_error(
-                            format!("Could not check for new updates: {}", err).as_str(),
-                        );
-                    }
-                }
-            } else {
-                info!("Check for updates is disabled");
-            }
-        }
-    }
-
-    /// ### context
-    ///
     /// Returns a reference to context
     fn context(&self) -> &Context {
         self.context.as_ref().unwrap()
     }
 
-    /// ### context_mut
-    ///
     /// Returns a mutable reference to context
     fn context_mut(&mut self) -> &mut Context {
         self.context.as_mut().unwrap()
     }
 
-    /// ### theme
-    ///
+    /// Returns config client reference
+    fn config(&self) -> &ConfigClient {
+        self.context().config()
+    }
+
     /// Returns a reference to theme
     fn theme(&self) -> &Theme {
         self.context().theme_provider().theme()
     }
+
+    /// Get current input mask to show
+    fn input_mask(&self) -> InputMask {
+        match self.protocol {
+            FileTransferProtocol::AwsS3 => InputMask::AwsS3,
+            FileTransferProtocol::Ftp(_)
+            | FileTransferProtocol::Scp
+            | FileTransferProtocol::Sftp => InputMask::Generic,
+        }
+    }
 }
 
 impl Activity for AuthActivity {
-    /// ### on_create
-    ///
     /// `on_create` is the function which must be called to initialize the activity.
     /// `on_create` must initialize all the data structures used by the activity
     /// Context is taken from activity manager and will be released only when activity is destroyed
@@ -178,9 +247,11 @@ impl Activity for AuthActivity {
         // Set context
         self.context = Some(context);
         // Clear terminal
-        self.context_mut().clear_screen();
+        if let Err(err) = self.context_mut().terminal().clear_screen() {
+            error!("Failed to clear screen: {}", err);
+        }
         // Put raw mode on enabled
-        if let Err(err) = enable_raw_mode() {
+        if let Err(err) = self.context_mut().terminal().enable_raw_mode() {
             error!("Failed to enter raw mode: {}", err);
         }
         // If check for updates is enabled, check for updates
@@ -201,8 +272,6 @@ impl Activity for AuthActivity {
         info!("Activity initialized");
     }
 
-    /// ### on_draw
-    ///
     /// `on_draw` is the function which draws the graphical interface.
     /// This function must be called at each tick to refresh the interface
     fn on_draw(&mut self) {
@@ -210,29 +279,26 @@ impl Activity for AuthActivity {
         if self.context.is_none() {
             return;
         }
-        // Read one event
-        if let Ok(Some(event)) = self.context().input_hnd().read_event() {
-            // Set redraw to true
-            self.redraw = true;
-            // Handle on resize
-            if let Event::Resize(_, h) = event {
-                self.check_minimum_window_size(h);
+        // Tick
+        match self.app.tick(PollStrategy::UpTo(3)) {
+            Ok(messages) => {
+                for msg in messages.into_iter() {
+                    let mut msg = Some(msg);
+                    while msg.is_some() {
+                        msg = self.update(msg);
+                    }
+                }
+            }
+            Err(err) => {
+                self.mount_error(format!("Application error: {}", err));
             }
-            // Handle event on view and update
-            let msg = self.view.on(event);
-            self.update(msg);
         }
-        // Redraw if necessary
+        // View
         if self.redraw {
-            // View
             self.view();
-            // Set redraw to false
-            self.redraw = false;
         }
     }
 
-    /// ### will_umount
-    ///
     /// `will_umount` is the method which must be able to report to the activity manager, whether
     /// the activity should be terminated or not.
     /// If not, the call will return `None`, otherwise return`Some(ExitReason)`
@@ -240,24 +306,17 @@ impl Activity for AuthActivity {
         self.exit_reason.as_ref()
     }
 
-    /// ### on_destroy
-    ///
     /// `on_destroy` is the function which cleans up runtime variables and data before terminating the activity.
     /// This function must be called once before terminating the activity.
     /// This function finally releases the context
     fn on_destroy(&mut self) -> Option<Context> {
         // Disable raw mode
-        if let Err(err) = disable_raw_mode() {
+        if let Err(err) = self.context_mut().terminal().disable_raw_mode() {
             error!("Failed to disable raw mode: {}", err);
         }
-        self.context.as_ref()?;
-        // Clear terminal and return
-        match self.context.take() {
-            Some(mut ctx) => {
-                ctx.clear_screen();
-                Some(ctx)
-            }
-            None => None,
+        if let Err(err) = self.context_mut().terminal().clear_screen() {
+            error!("Failed to clear screen: {}", err);
         }
+        self.context.take()
     }
 }
