@@ -27,8 +27,5 @@
  */
 // Modules
 pub mod activities;
-pub(crate) mod components;
 pub mod context;
-pub(crate) mod input;
-pub(crate) mod keymap;
 pub(crate) mod store;
